@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  * 
- * @(#) $Id: disman-script-mib-proc.h 709 2004-10-14 20:11:17Z schoenw $
+ * @(#) $Id: disman-script-mib-proc.h 3180 2009-07-29 22:36:13Z schoenw $
  */
 
 #ifndef _DISMAN_SCRIPT_MIB_PROC_H_
@@ -36,13 +36,15 @@ extern void
 disman_script_mib_proc_create_script(GNetSnmp *s,
 				     guchar *owner,
 				     guchar *name,
-				     guchar *descr);
+				     guchar *descr,
+				     GError **error);
 
 extern void
 disman_script_mib_proc_create_run(GNetSnmp *s,
 				  guchar *lowner,
 				  guchar *lname,
-				  guchar *args);
+				  guchar *args,
+				  GError **error);
 
 
 #endif /* _DISMAN_SCRIPT_MIB_PROC_H_ */
