@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  * 
- * @(#) $Id: if-mib-proc.h 709 2004-10-14 20:11:17Z schoenw $
+ * @(#) $Id: if-mib-proc.h 3180 2009-07-29 22:36:13Z schoenw $
  */
 
 #ifndef _IF_MIB_PROC_H_
@@ -36,7 +36,8 @@ extern void
 if_mib_proc_get_ifTable(GNetSnmp *s,
 			if_mib_ifEntry_t ***ifEntry,
 			gint mask,
-			time_t epoch);
+			time_t epoch,
+			GError **error);
 
 extern void
 if_mib_proc_free_ifTable(if_mib_ifEntry_t **ifEntry);
