@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  * 
- * @(#) $Id: productmib-proc.h 709 2004-10-14 20:11:17Z schoenw $
+ * @(#) $Id: productmib-proc.h 3180 2009-07-29 22:36:13Z schoenw $
  */
 
 #ifndef _PRODUCTMIB_PROC_H_
@@ -27,12 +27,14 @@
 
 extern void
 productmib_proc_create_vlan(GNetSnmp *s, gint32 vlanid,
-			    guchar *name, gsize name_len, guint32 type);
+			    guchar *name, gsize name_len, guint32 type,
+			    GError **error);
 extern void
-productmib_proc_delete_vlan(GNetSnmp *s, gint32 ifIndex);
+productmib_proc_delete_vlan(GNetSnmp *s, gint32 ifIndex, GError **error);
 
 extern void
 productmib_proc_set_vlan_port_member(GNetSnmp *s, gint32 ifIndex,
-				     guchar *ports, gsize ports_len);
+				     guchar *ports, gsize ports_len,
+				     GError **error);
     
 #endif /* _PRODUCTMIB_PROC_H_ */
