@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  * 
- * @(#) $Id: snmp-user-based-sm-mib-proc.h 709 2004-10-14 20:11:17Z schoenw $
+ * @(#) $Id: snmp-user-based-sm-mib-proc.h 3180 2009-07-29 22:36:13Z schoenw $
  */
 
 #ifndef _SNMP_USER_BASED_SM_MIB_PROC_H_
@@ -29,13 +29,15 @@
 extern void
 snmp_user_based_sm_mib_proc_clone_user(GNetSnmp *s,
 				       guchar *user, size_t user_len,
-				       guchar *dolly, size_t dolly_len);
+				       guchar *dolly, size_t dolly_len,
+				       GError **error);
 
 extern void
 snmp_user_based_sm_mib_proc_delete_user(GNetSnmp *s,
 					guchar *name, gsize name_len,
 					guchar *group, gsize group_len,
-					gint32 model);
+					gint32 model,
+					GError **error);
 
 
 #endif /* _SNMP_USER_BASED_SM_MIB_PROC_H_ */
