@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  * 
- * @(#) $Id: udp-mib-proc.c 709 2004-10-14 20:11:17Z schoenw $
+ * @(#) $Id: udp-mib-proc.c 3180 2009-07-29 22:36:13Z schoenw $
  */
 
 #include <stdio.h>
@@ -36,15 +36,16 @@ struct state {
 
 extern void
 udp_mib_proc_get_stats(GNetSnmp *s,
-		       udp_mib_proc_stats_t **udpStatsEntry)
+		       udp_mib_proc_stats_t **udpStatsEntry,
+		       GError **error)
 {
     udp_mib_udp_t *udp;
     static struct state state = { {0,0}, NULL};
     GTimeVal now;
     double delta;
     
-    udp_mib_get_udp(s, &udp, 0);
-    if (s->error_status || !udp) return;
+    udp_mib_get_udp(s, &udp, 0, error);
+    if ((error && *error) || s->error_status || !udp) return;
 
 #if 0
     if (state.last_poll_time.tv_sec < interp->epoch) {
