@@ -1,7 +1,7 @@
 /* 
  * basic.c -- basic functions for the scli command interpreter
  *
- * Copyright (C) 2001 Juergen Schoenwaelder
+ * Copyright (C) 2001-2009 Juergen Schoenwaelder
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  * 
- * @(#) $Id: basic.c 2111 2007-01-02 23:46:40Z schoenw $
+ * @(#) $Id: basic.c 3260 2010-02-19 08:18:43Z schoenw $
  */
 
 #include "scli.h"
@@ -34,7 +34,9 @@
 #endif
 
 
-char const scli_copyright[] = "(c) 2001-2007 Juergen Schoenwaelder";
+char const scli_copyright[] = "(c) 2001-2010 Juergen Schoenwaelder";
+
+GStaticMutex scli_global_lock = G_STATIC_MUTEX_INIT;
 
 static int scli_curses_running = 0;
 
@@ -239,7 +241,7 @@ page(scli_interp_t *interp, GString *s)
 	return;
     }
     
-    if (! interp->pager) {
+    if (scli_interp_proto(interp) || !interp->pager) {
 	goto nopager;
     }
 
@@ -283,14 +285,9 @@ scli_interp_create(char *name)
     interp->header = g_string_new(NULL);
     interp->epoch = time(NULL);
     interp->regex_flags = REG_EXTENDED|REG_NOSUB;
-    interp->tdomain = GNET_SNMP_TDOMAIN_UDP_IPV4;
-    interp->taddress = gnet_inetaddr_new("localhost", 161);
     interp->retries = gnet_snmp_retries;
     interp->timeout = gnet_snmp_timeout;
     interp->snmp = gnet_snmp_version;
-#if 0
-    interp->tdomain = gnet_snmp_transport; /* ?? */
-#endif
 
     return interp;
 }
@@ -313,6 +310,9 @@ scli_interp_delete(scli_interp_t *interp)
 	if (interp->result) {
 	    g_string_free(interp->result, 1);
 	}
+	if (interp->error) {
+	    g_clear_error(&interp->error);
+	}
 	if (interp->header) {
 	    g_string_free(interp->header, 1);
 	}
@@ -349,6 +349,7 @@ void scli_interp_init(scli_interp_t *interp)
     scli_init_netsnmp_mode(interp);
     scli_init_3com_mode(interp);
     scli_init_hp_mode(interp);
+    scli_init_sflow_mode(interp);
 }
 
 
@@ -361,11 +362,47 @@ scli_interp_reset(scli_interp_t *interp)
     if (interp->xml_doc) {
 	xmlFreeDoc(interp->xml_doc);
     }
-    interp->xml_doc = xmlNewDoc("1.0");
+    if (interp->error) {
+	g_clear_error(&interp->error);
+    }
+    interp->xml_doc = xmlNewDoc(BAD_CAST("1.0"));
     interp->xml_doc->children = xmlNewDocNode(interp->xml_doc, NULL,
-					      "scli", NULL);
+					      BAD_CAST("scli"), NULL);
     interp->xml_node = interp->xml_doc->children;
-    xmlSetProp(interp->xml_node, "version", VERSION);
+    xml_set_prop(interp->xml_node, BAD_CAST("version"), "%s", VERSION);
+}
+
+
+
+gboolean
+scli_interp_set_error(scli_interp_t *interp, GError **error)
+{
+    if (error && *error) {
+	interp->error = g_error_copy(*error);
+	g_clear_error(error);
+	return TRUE;
+    }
+
+    return FALSE;
+}
+
+
+
+gboolean
+scli_interp_set_error_snmp(scli_interp_t *interp, GError **error)
+{
+    if (scli_interp_set_error(interp, error)) {
+#if (GLIB_MINOR_VERSION > 15)
+	g_prefix_error(&interp->error, "snmp: ");
+#endif
+	return TRUE;
+    }
+    if (interp->peer->error_status) {
+	/* in the future, we might generate a proper GError for this */
+	return TRUE;
+    }
+
+    return FALSE;
 }
 
 
@@ -697,11 +734,11 @@ get_xml_tree(scli_interp_t *interp, char *xpath)
     tree = xmlDocGetRootElement(interp->xml_doc);
     for (i = 0; argv[i]; i++) {
 	for (sibling = tree->children; sibling; sibling = sibling->next) {
-	    if (strcmp(sibling->name, argv[i]) == 0) {
+	    if (strcmp((char *) sibling->name, argv[i]) == 0) {
 		break;
 	    }
 	}
-	tree = sibling ? sibling : xmlNewChild(tree, NULL, argv[i], NULL);
+	tree = sibling ? sibling : xml_new_child(tree, NULL, BAD_CAST(argv[i]), NULL);
     }
 
     g_strfreev(argv);
@@ -800,7 +837,7 @@ show_result(scli_interp_t *interp, int code)
     if (scli_interp_xml(interp) && !(interp->flags & SCLI_INTERP_FLAG_FMT)) {
 	xmlDocDumpFormatMemory(interp->xml_doc, &buffer, &len, 1);
 	g_string_truncate(interp->header, 0);
-	g_string_assign(interp->result, buffer);
+        g_string_assign(interp->result, (gchar *) buffer);
 	xmlFree(buffer);
     }
     if (! scli_interp_recursive(interp) && interp->header->len) {
@@ -822,7 +859,7 @@ scli_show_results(scli_interp_t *interp, scli_cmd_t *cmd, int code)
 	if (error_infos[i].code == code) break;
     }
     g_assert(error_infos[i].code);
-    
+
     switch (code) {
     case SCLI_SYNTAX_VALUE:
     case SCLI_SYNTAX_NUMBER:
@@ -846,7 +883,9 @@ scli_show_results(scli_interp_t *interp, scli_cmd_t *cmd, int code)
 				 cmd->options ? cmd->options : "");
 	break;
     case SCLI_SNMP:
-	if (interp->peer) {
+	if (interp->error && interp->error->message) {
+	    reason = g_strdup(interp->error->message);
+	} else if (interp->peer) {
 	    const char *error;
 	    error = gnet_snmp_enum_get_label(gnet_snmp_enum_error_table,
 					     interp->peer->error_status);
@@ -893,15 +932,13 @@ scli_show_results(scli_interp_t *interp, scli_cmd_t *cmd, int code)
 	    }
 #endif
 	    if (interp->peer) {
-		GURI *uri;
-		uri = gnet_snmp_get_uri(interp->peer);
-		if (uri) {
-		    gchar *name = gnet_uri_get_string(uri);
-		    xml_set_prop(top, "uri", name ? name : "?");
+		gchar *name = gnet_snmp_get_uri_string(interp->peer);
+		if (name) {
+		    xml_set_prop(top, BAD_CAST("uri"), name ? name : "?");
 		    g_free(name);
 		}
 	    }
-	    xml_set_prop(top, "date", "%s", xml_timeticks(0));
+	    xml_set_prop(top, BAD_CAST("date"), "%s", xml_timeticks(0));
 	}
 	if (! (scli_interp_recursive(interp))) {
 	    if (interp->xml_doc->children
@@ -928,7 +965,10 @@ scli_show_results(scli_interp_t *interp, scli_cmd_t *cmd, int code)
 		}
 	    }
 	} else {
-	    g_printerr("%3d %s\n", code, reason);
+	    if (interp->master && interp->name) {
+		g_print("%s ", interp->name);
+	    }
+	    g_print("%3d %s\n", code, reason);
 	}
     }
 
@@ -969,7 +1009,7 @@ eval_all_cmd_node(scli_interp_t *interp, GNode *node, GString *s)
 			if (cmd->flags & SCLI_CMD_FLAG_NEED_PEER) {
 			    gchar *name = NULL;
 			    if (interp->peer) {
-				name = gnet_inetaddr_get_canonical_name(interp->peer->taddress);
+				name = gnet_snmp_taddress_get_short_name(interp->peer->taddress);
 			    }
 			    g_string_sprintfa(s, " [%s]", (name) ? name : "?");
 			    g_string_sprintfa(s, " [%s]", fmt_timeticks(0));
@@ -999,6 +1039,12 @@ eval_all_cmd_node(scli_interp_t *interp, GNode *node, GString *s)
 }
 
 
+/*
+ * This is the most basic command evaluation function which assumes
+ * the caller already knows which scli_cmt_t needs to be called.
+ * Please always use this function and never bypass it by calling
+ * cmd->func directly.
+ */
 
 int
 scli_eval_cmd(scli_interp_t *interp, scli_cmd_t *cmd, int argc, char **argv)
@@ -1007,6 +1053,10 @@ scli_eval_cmd(scli_interp_t *interp, scli_cmd_t *cmd, int argc, char **argv)
 
     g_return_val_if_fail(cmd && cmd->func, SCLI_OK);
 
+#if 1
+    g_static_mutex_lock(&scli_global_lock);
+#endif
+
     g_string_truncate(interp->result, 0);
     g_string_truncate(interp->header, 0);
     if (! scli_interp_dry(interp)
@@ -1020,7 +1070,8 @@ scli_eval_cmd(scli_interp_t *interp, scli_cmd_t *cmd, int argc, char **argv)
 	    code = SCLI_ERROR_NOXML;
 	    goto done;
 	} else {
-	    return SCLI_OK;
+	    code = SCLI_OK;
+	    goto quit;
 	}
     }
 
@@ -1051,10 +1102,17 @@ scli_eval_cmd(scli_interp_t *interp, scli_cmd_t *cmd, int argc, char **argv)
 
  done:
     scli_show_results(interp, cmd, code);
+ quit:
+#if 1
+    g_static_mutex_unlock(&scli_global_lock);
+#endif
     return code;
 }
 
-
+/*
+ * This is the most basic evaluation function for string arguments (I
+ * am not counting scli_eval_cmd() here).
+ */
 
 int
 scli_eval_argc_argv(scli_interp_t *interp, int argc, char **argv)
@@ -1062,26 +1120,10 @@ scli_eval_argc_argv(scli_interp_t *interp, int argc, char **argv)
     GNode *node = NULL;
     gboolean done = FALSE;
     int i, code = SCLI_OK;
-#ifdef SCLI_FORK
-    pid_t pid = -1;
-#endif
 
     scli_interp_reset(interp);
     interp->xid++;
 
-#ifdef SCLI_FORK
-    pid = fork();
-    if (pid < 0) {
-	    /* xxxx */
-    }
-    if (pid == 0) {
-	    g_print("** %d **\n", getpid());
-    } else {
-	    waitpid(pid, &code, 0);
-	    return code;
-    }
-#endif
-
     node = g_node_first_child(interp->cmd_root);
     for (i = 0; i < argc && ! done; i++) {
 	while (node) {
@@ -1121,7 +1163,7 @@ scli_eval_argc_argv(scli_interp_t *interp, int argc, char **argv)
 	    g_string_free(s, 1);
 	    show_result(interp, code);
 	    if (scli_interp_proto(interp)) {
-		g_printerr("%3d xxx\n", code);
+		g_print("%3d xxx\n", code);
 	    }
 	    scli_interp_reset(interp);
 	    interp->flags &= ~SCLI_INTERP_FLAG_RECURSIVE;
@@ -1130,6 +1172,9 @@ scli_eval_argc_argv(scli_interp_t *interp, int argc, char **argv)
 
     if (! done) {
 	int j;
+	if (interp->master && interp->name) {
+	    g_print("%s ", interp->name);
+	}
 	g_print("%3d invalid command name \"", SCLI_SYNTAX_COMMAND);
 	for (j = 0; j <= i; j++) {
 	    g_print("%s%s", j ? " ": "", argv[j]);
@@ -1138,10 +1183,6 @@ scli_eval_argc_argv(scli_interp_t *interp, int argc, char **argv)
 	code = SCLI_SYNTAX;
     }
 
-#ifdef SCLI_FORK
-    exit(code);
-#endif
-
     return code;
 }
 
@@ -1281,11 +1322,16 @@ scli_eval_init_file(scli_interp_t *interp)
 
 
 int
-scli_open_community(scli_interp_t *interp, GNetSnmpTDomain tdomain,
-		    GInetAddr *taddress, char *community)
+#if 0
+scli_open_community(scli_interp_t *interp, GNetSnmpTAddress *taddress,
+		    char *community)
+#else
+scli_open_community(scli_interp_t *interp, gchar *target)
+#endif
 {
     snmpv2_mib_system_t *system = NULL;
     snmpv2_mib_snmpSet_t *serial = NULL;
+    GError *error = NULL;
     int verbose;
     const char *transport_domain = NULL;
 
@@ -1293,8 +1339,9 @@ scli_open_community(scli_interp_t *interp, GNetSnmpTDomain tdomain,
 	scli_close(interp);
     }
 
+#if 0
     interp->peer = gnet_snmp_new();
-    gnet_snmp_set_transport(interp->peer, tdomain, taddress);
+    gnet_snmp_set_transport(interp->peer, taddress);
     gnet_snmp_set_community(interp->peer, community ? community : "public");
     gnet_snmp_set_timeout(interp->peer, interp->timeout);
     gnet_snmp_set_retries(interp->peer, interp->retries);
@@ -1302,14 +1349,22 @@ scli_open_community(scli_interp_t *interp, GNetSnmpTDomain tdomain,
     if (! interp->peer) {
 	return SCLI_SNMP_NAME;
     }
+#else
+    interp->peer = gnet_snmp_new_string(target, &error);
+    if (! interp->peer) {
+	scli_interp_set_error(interp, &error);
+	return SCLI_SNMP;
+    }
+#endif
 
-    if (tdomain == GNET_SNMP_TDOMAIN_TCP_IPV6
-	|| tdomain == GNET_SNMP_TDOMAIN_TCP_IPV4) {
+    if (interp->peer->taddress->domain == GNET_SNMP_TDOMAIN_TCP_IPV6
+	|| interp->peer->taddress->domain == GNET_SNMP_TDOMAIN_TCP_IPV4
+	|| interp->peer->taddress->domain == GNET_SNMP_TDOMAIN_LOCAL) {
 	gnet_snmp_set_retries(interp->peer, 0);
     }
 
     transport_domain = gnet_snmp_enum_get_label(gnet_snmp_enum_tdomain_table,
-						tdomain);
+					gnet_snmp_get_tdomain(interp->peer));
     
     /*
      * Lets see how we can talk to this guy. We first try to speek
@@ -1327,8 +1382,8 @@ scli_open_community(scli_interp_t *interp, GNetSnmpTDomain tdomain,
 	|| interp->snmp == GNET_SNMP_V2C
 	|| interp->snmp == GNET_SNMP_V3) {
 	interp->peer->version = interp->snmp;
-	snmpv2_mib_get_system(interp->peer, &system, SNMPV2_MIB_SYSUPTIME);
-	if (interp->peer->error_status) {
+	snmpv2_mib_get_system(interp->peer, &system, SNMPV2_MIB_SYSUPTIME, &error);
+	if (scli_interp_set_error_snmp(interp, &error)) {
 	    scli_close(interp);
 	}
     } else {
@@ -1338,8 +1393,8 @@ scli_open_community(scli_interp_t *interp, GNetSnmpTDomain tdomain,
 		    transport_domain ? transport_domain : "");
 	}
 	interp->peer->version = GNET_SNMP_V2C;
-	snmpv2_mib_get_system(interp->peer, &system, SNMPV2_MIB_SYSUPTIME);
-	if (interp->peer->error_status == 0) {
+	snmpv2_mib_get_system(interp->peer, &system, SNMPV2_MIB_SYSUPTIME, &error);
+	if (! scli_interp_set_error_snmp(interp, &error)) {
 	    if (verbose) {
 		g_print("good\n");
 	    }
@@ -1350,8 +1405,9 @@ scli_open_community(scli_interp_t *interp, GNetSnmpTDomain tdomain,
 			transport_domain ? transport_domain : "");
 	    }
 	    interp->peer->version = GNET_SNMP_V1;
-	    snmpv2_mib_get_system(interp->peer, &system, SNMPV2_MIB_SYSUPTIME);
-	    if (interp->peer->error_status == 0) {
+	    g_clear_error(&error);
+	    snmpv2_mib_get_system(interp->peer, &system, SNMPV2_MIB_SYSUPTIME, &error);
+	    if (! scli_interp_set_error_snmp(interp, &error)) {
 		if (verbose) {
 		    g_print("ok.\n");
 		}
@@ -1371,6 +1427,9 @@ scli_open_community(scli_interp_t *interp, GNetSnmpTDomain tdomain,
 	return SCLI_SNMP;
     }
 
+    g_clear_error(&error);
+
+#if 0
     if (verbose) {
 	g_print("%3d scli trying SNMP advisory lock support ... ", SCLI_MSG);
     }
@@ -1389,6 +1448,7 @@ scli_open_community(scli_interp_t *interp, GNetSnmpTDomain tdomain,
     if (serial) {
 	snmpv2_mib_free_snmpSet(serial);
     }
+#endif
     
     interp->epoch = time(NULL);
     return SCLI_OK;
@@ -1421,7 +1481,7 @@ scli_prompt(scli_interp_t *interp)
     
     if (interp->peer) {
 	prompt = g_strdup_printf("(%s) scli > ",
-		 gnet_inetaddr_get_name(interp->peer->taddress));
+		 gnet_snmp_taddress_get_short_name(interp->peer->taddress));
     } else {
 	prompt = g_strdup_printf("scli > ");
     }
