@@ -1,7 +1,7 @@
 /* 
  * cmds.c -- basic commands for the scli command interpreter
  *
- * Copyright (C) 2001 Juergen Schoenwaelder
+ * Copyright (C) 2001-2009 Juergen Schoenwaelder
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  * 
- * @(#) $Id: cmds.c 2111 2007-01-02 23:46:40Z schoenw $
+ * @(#) $Id: cmds.c 3258 2010-02-17 12:19:35Z schoenw $
  */
 
 #include "scli.h"
@@ -50,12 +50,42 @@ alias_compare(gconstpointer a, gconstpointer b)
 static int
 cmd_scli_exit(scli_interp_t *interp, int argc, char **argv)
 {
+    GSList *elem;
+    scli_interp_t *slave = NULL;
+
     g_return_val_if_fail(interp, SCLI_ERROR);
         
     if (argc > 1) {
 	return SCLI_SYNTAX_NUMARGS;
     }
 
+    /*
+     * Wait until all threads have finished they command queues.  The
+     * clean way would be to ship an exit command to all threads and
+     * then to wait until the threads terminate themself.
+     */
+
+    for (elem = interp->slave_list; elem; elem = g_slist_next(elem)) {
+	slave = (scli_interp_t *) elem->data;
+	if (slave->thread) {
+	    char *cmd_exit = "exit";
+	    g_print("xxx pushing exit %p\n", slave->thread);
+	    scli_thread_eval_argc_argv(slave, 1, &cmd_exit);
+	}
+    }
+	    
+ again:
+    for (elem = interp->slave_list; elem; elem = g_slist_next(elem)) {
+	slave = (scli_interp_t *) elem->data;
+	if (slave->thread) {
+	    g_print("xxx joining %p\n", slave->thread);
+	    (void) g_thread_join(slave->thread);
+	    g_print("xxx joining %p done\n", slave->thread);
+	    slave->thread = NULL;
+	    goto again;
+	}
+    }
+
     return SCLI_EXIT;
 }
 
@@ -472,10 +502,12 @@ scli_cmd_open(scli_interp_t *interp, int argc, char **argv)
     gint ipv6, port = 161;
     gchar *community = NULL;
     GInetAddr *addr;
+    GNetSnmpTAddress *taddr;
     int code;
     
     g_return_val_if_fail(interp, SCLI_ERROR);
 
+#if 0
     if (argc < 2 || argc > 4) {
         return SCLI_SYNTAX_NUMARGS;
     }
@@ -489,18 +521,21 @@ scli_cmd_open(scli_interp_t *interp, int argc, char **argv)
 	community = argv[3];
     }
 
+    taddr = gnet_snmp_trans
+
     addr = gnet_inetaddr_new(host, port);
     if (! addr) {
 	return SCLI_SYNTAX_VALUE;
     }
 
-    if (interp->taddress) gnet_inetaddr_delete(interp->taddress);
-    interp->taddress = addr;
-    ipv6 = gnet_inetaddr_is_ipv6(interp->taddress);
+    ipv6 = gnet_inetaddr_is_ipv6(addr);
 
     /*
      * We prefer to use TCP so try TCP first and fall back to UDP if
-     * this fails for whatever reason.
+     * this fails for whatever reason. Note that this may lead to a
+     * long long TCP timeout. Should we have an option to force the
+     * transport domain? Yes, lets add an scli configuration property
+     * which lists the sequence of transports to probe.
      */
 #if 1
     interp->tdomain
@@ -516,6 +551,12 @@ scli_cmd_open(scli_interp_t *interp, int argc, char **argv)
 	    = ipv6 ? GNET_SNMP_TDOMAIN_UDP_IPV6 : GNET_SNMP_TDOMAIN_UDP_IPV4;
     code = scli_open_community(interp, interp->tdomain,
 			       interp->taddress, community);
+#else
+    if (argc != 2) {
+        return SCLI_SYNTAX_NUMARGS;
+    }
+    code = scli_open_community(interp, argv[1]);
+#endif
     return code;
 }
 
@@ -747,7 +788,13 @@ scli_interp_eval(scli_interp_t *interp, int argc, char **argv)
 	return SCLI_ERROR;
     }
 
-    return scli_eval_argc_argv(slave, argc-1, argv+1);
+    if (slave->thread) {
+	(void) scli_thread_eval_argc_argv(slave, argc-1, argv+1);
+    } else {
+	(void) scli_eval_argc_argv(slave, argc-1, argv+1);
+    }
+
+    return SCLI_OK;
 }
 
 
@@ -792,6 +839,9 @@ create_scli_interp(scli_interp_t *interp, int argc, char **argv)
     cmd->func = scli_interp_eval;
     scli_create_command(interp, cmd);
 
+    slave->cmd_queue = g_async_queue_new();
+    slave->thread = g_thread_create(scli_thread, slave, TRUE, NULL);
+
     return SCLI_OK;
 }
 
@@ -895,14 +945,18 @@ show_scli_info(scli_interp_t *interp, int argc, char **argv)
 		      "Pager:", interp->pager ? interp->pager : "scli");
 
     if (interp->peer) {
-        GURI *uri;
-	uri = gnet_snmp_get_uri(interp->peer);
-	if (uri) {
-	    gchar *name = gnet_uri_get_string(uri);
+	gchar *name = gnet_snmp_get_uri_string(interp->peer);
+	if (name) {
 	    g_string_sprintfa(interp->result, "%-*s %s\n",
 			      indent, "URI:", name);
 	    g_free(name);
 	}
+	label = gnet_snmp_enum_get_label(gnet_snmp_enum_tdomain_table,
+				 (gint32) gnet_snmp_get_tdomain(interp->peer));
+	if (label) {
+	    g_string_sprintfa(interp->result, "%-*s %s\n", indent,
+			      "Transport:", label);
+	}
 	g_string_sprintfa(interp->result, "%-*s %u\n", indent,
 			  "Retries:", gnet_snmp_get_retries(interp->peer));
 	g_string_sprintfa(interp->result, "%-*s %u ms\n", indent,
@@ -1255,6 +1309,7 @@ cmd_scli_walk(scli_interp_t *interp, int argc, char **argv)
     GList *elem;
     int i, len;
     char *p;
+    GError *error = NULL;
     GNetSnmpVarBind *vb;
 
     g_return_val_if_fail(interp, SCLI_ERROR);
@@ -1288,9 +1343,13 @@ cmd_scli_walk(scli_interp_t *interp, int argc, char **argv)
 	in = g_list_append(in, vb);
     }
 
-    out = gnet_snmp_sync_walk(interp->peer, in);
+    out = gnet_snmp_sync_walk(interp->peer, in, &error);
     g_list_foreach(in, (GFunc) gnet_snmp_varbind_delete, NULL);
     g_list_free(in);
+    if (error) {
+	g_string_assign(interp->result, error->message);
+	return SCLI_ERROR;
+    }
     if (out) {
 	for (elem = out; elem; elem = g_list_next(elem)) {
 	    fmt_walk(interp->result, (GNetSnmpVarBind *) elem->data);
@@ -1305,8 +1364,7 @@ cmd_scli_walk(scli_interp_t *interp, int argc, char **argv)
 
 
 struct scan_elem {
-    GNetSnmpTDomain tdomain;
-    GInetAddr *taddress;	/* xxx - not very efficient */
+    GNetSnmpTAddress *taddress;	/* xxx - not very efficient */
     int descr_len;
     char *descr;
 };
@@ -1316,7 +1374,7 @@ scan_elem_free(gpointer data, gpointer user_data)
 {
     struct scan_elem *p = data;
     if (p) {
-	if (p->taddress) gnet_inetaddr_delete(p->taddress);
+	if (p->taddress) gnet_snmp_taddress_delete(p->taddress);
 	if (p->descr) g_free(p->descr);
 	g_free(p);
     }
@@ -1373,11 +1431,11 @@ scan_cmp(gconstpointer a, gconstpointer b)
     gchar ip_b[GNET_INETADDR_MAX_LEN];
     gint len_a, len_b;
 
-    len_a = gnet_inetaddr_get_length(n_a->taddress);
-    len_b = gnet_inetaddr_get_length(n_b->taddress);
+    len_a = gnet_inetaddr_get_length(n_a->taddress->inetaddr);
+    len_b = gnet_inetaddr_get_length(n_b->taddress->inetaddr);
 
-    gnet_inetaddr_get_bytes(n_a->taddress, ip_a);
-    gnet_inetaddr_get_bytes(n_b->taddress, ip_b);
+    gnet_inetaddr_get_bytes(n_a->taddress->inetaddr, ip_a);
+    gnet_inetaddr_get_bytes(n_b->taddress->inetaddr, ip_b);
 
     if (len_a == len_b) {
 	return memcmp(ip_a, ip_b, len_a);
@@ -1393,7 +1451,7 @@ scan_print(gpointer data, gpointer user_data)
     scli_interp_t *interp = (scli_interp_t *) user_data;
     gchar *name = NULL;
 
-    name = gnet_inetaddr_get_canonical_name(p->taddress);
+    name = gnet_snmp_taddress_get_short_name(p->taddress);
     g_string_sprintfa(interp->result, "%-16s ", name);
     if (p->descr_len && p->descr) {
 	append_flat_string(interp->result, p->descr_len, p->descr);
@@ -1413,8 +1471,7 @@ scan_one_done(GNetSnmp *s, GNetSnmpPdu *pdu, GList *vbl, gpointer magic)
     if (s->error_status == GNET_SNMP_PDU_ERR_NOERROR && vbl) {
 	struct scan_elem *p;
 	p = g_new0(struct scan_elem, 1);
-	p->tdomain = s->tdomain;
-	p->taddress = gnet_inetaddr_clone(s->taddress);
+	p->taddress = gnet_snmp_taddress_clone(s->taddress);
 	vb = vbl->data;
 	if (vb->type == GNET_SNMP_VARBIND_TYPE_OCTETSTRING) {
 	    p->descr_len = vb->value_len;
@@ -1438,45 +1495,60 @@ scan_one_time(GNetSnmp *s, void *magic)
 static void
 scan_one(char *host, int port, char *community, gpointer magic)
 {
-    GInetAddr *taddress;
+    GInetAddr *inetaddr;
     GNetSnmpTDomain tdomain;
+    GNetSnmpTAddress *taddress;
     GNetSnmp *s;
     static GList *in = NULL;
     static guint32 base[] = {1, 3, 6, 1, 2, 1, 1, 1, 0};
     GNetSnmpVarBind *vb;
+    GError *error = NULL;
 
-    taddress = gnet_inetaddr_new(host, port); /* xxx where to release? */
-    if (! taddress) {
+    inetaddr = gnet_inetaddr_new(host, port); /* xxx where to release? */
+    if (! inetaddr) {
         g_warning("failed to convert address");
 	return;
     }
-    tdomain = gnet_inetaddr_is_ipv6(taddress)
-	    ? GNET_SNMP_TDOMAIN_UDP_IPV6 : GNET_SNMP_TDOMAIN_UDP_IPV4;
-     
+
+    taddress = gnet_snmp_taddress_new_inet(
+    	gnet_inetaddr_is_ipv6(inetaddr) ? GNET_SNMP_TDOMAIN_UDP_IPV6 : GNET_SNMP_TDOMAIN_UDP_IPV4, inetaddr);
+    if (! taddress) {
+        g_warning("failed to generate transport address");
+	return;
+    }
+
     s = gnet_snmp_new();
-    gnet_snmp_set_transport(s, tdomain, taddress);
+    gnet_snmp_set_transport(s, taddress);
     gnet_snmp_set_community(s, community ? community : "public");
     gnet_snmp_set_version(s, GNET_SNMP_V1);
     if (! s) {
-	 g_warning("failed to create session: %s\n", host);
-	 return;
+	g_warning("failed to create session: %s\n", host);
+	if (taddress) gnet_snmp_taddress_delete(taddress);
+	if (inetaddr) gnet_inetaddr_delete(inetaddr);
+	return;
     }
 
     if (! in) {
-	 vb = gnet_snmp_varbind_new(base, G_N_ELEMENTS(base),
-				    GNET_SNMP_VARBIND_TYPE_NULL, NULL, 0);
-	 in = g_list_append(in, vb);
+	vb = gnet_snmp_varbind_new(base, G_N_ELEMENTS(base),
+				   GNET_SNMP_VARBIND_TYPE_NULL, NULL, 0);
+	in = g_list_append(in, vb);
     }
 
     s->done_callback = scan_one_done;
     s->time_callback = scan_one_time;
     s->magic = magic;
 
-    if (gnet_snmp_async_get(s, in) == NULL) {
-        g_warning("failed to send async request");
+    if (gnet_snmp_async_get(s, in, &error) == NULL) {
+        g_warning("failed to send async request%s%s",
+		  error ? ": " : "",
+		  error ? error->message : "");
+	if (taddress) gnet_snmp_taddress_delete(taddress);
+	if (inetaddr) gnet_inetaddr_delete(inetaddr);
 	return;
     }
     ((struct scanmagic *) magic)->count++;
+    if (taddress) gnet_snmp_taddress_delete(taddress);
+    if (inetaddr) gnet_inetaddr_delete(inetaddr);
 }
 
 
@@ -1574,6 +1646,29 @@ cmd_scli_scan(scli_interp_t *interp, int argc, char **argv)
 }
 
 
+int
+cmd_scli_sleep(scli_interp_t *interp, int argc, char **argv)
+{
+    char *end;
+    guint32 secs;
+    
+    g_return_val_if_fail(interp, SCLI_ERROR);
+
+    if (argc != 2) {
+	return SCLI_SYNTAX_NUMARGS;
+    }
+
+    secs = strtoul(argv[1], &end, 0);
+    if (*end || secs < 0) {
+	g_string_assign(interp->result, argv[1]);
+	return SCLI_SYNTAX_NUMBER;
+    }
+
+    sleep(secs);
+
+    return SCLI_OK;
+}
+
 
 void
 scli_init_scli_mode(scli_interp_t *interp)
@@ -1624,6 +1719,13 @@ scli_init_scli_mode(scli_interp_t *interp)
 	  NULL, NULL,
 	  cmd_scli_scan },
 
+	{ "run scli sleep", "<secs>",
+	  "The `run scli sleep' command simply sleeps for the given amount\n"
+	  "of seconds.",
+	  SCLI_CMD_FLAG_NORECURSE,
+	  NULL, NULL,
+	  cmd_scli_sleep },
+
 	{ "create scli plugin", "<module>",
 	  "The `create scli plugin' command dynamically loads an scli mode\n"
 	  "into a running scli process. This can be used to dynamically\n"
@@ -1776,17 +1878,17 @@ scli_init_scli_mode(scli_interp_t *interp)
 	  NULL, NULL,
 	  show_scli_info },
 
-	{ "show scli command info", "[<regex]",
+	{ "show scli command info", "[<regex>]",
 	  "The `show scli command info' command displays summary information\n"
-	  "about scli commands. The optional regular expression <regexp> is\n"
+	  "about scli commands. The optional regular expression <regex> is\n"
 	  "matched against the command names to select the scli commands.",
 	  SCLI_CMD_FLAG_NORECURSE,
 	  NULL, NULL,
 	  show_scli_command_info },
 
-	{ "show scli command details", "[<regex]",
+	{ "show scli command details", "[<regex>]",
 	  "The `show scli command details' command displays detailed information\n"
-	  "about scli commands. The optional regular expression <regexp> is\n"
+	  "about scli commands. The optional regular expression <regex> is\n"
 	  "matched against the command names to select the scli commands.",
 	  SCLI_CMD_FLAG_NORECURSE,
 	  NULL, NULL,
