@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  * 
- * @(#) $Id: vendors.c 742 2005-04-13 22:20:30Z schoenw $
+ * @(#) $Id: vendors.c 2887 2008-06-20 20:09:19Z schoenw $
  */
 
 #include "scli.h"
@@ -38,6 +38,7 @@ static scli_vendor_t iana_vendor_table[] = {
     {  480,	"QMS",			"http://www.qms.com/" },
     {  641,	"Lexmark",		"http://www.lexmark.com/" },
     {  762,	"Karlnet",		"http://www.karlnet.com/" },
+    {  890,	"ZyXEL",		"http://www.zyxel.com/" },
     {  915,	"SerComm",		"http://www.sercomm.com/" },
     {  937,	"Netgear",		"http://www.netgear.com/" },
     { 1347,	"Kyocera Corporation",	"http://www.kyocera.com/" },
@@ -52,6 +53,7 @@ static scli_vendor_t iana_vendor_table[] = {
     { 5567,	"Riverstone Networks",	"http://www.riverstonenet.com/" },
     { 7313,	"1stWave",              "http://www.1stwave.de/" },
     { 8072,	"NET-SNMP",		"http://www.net-snmp.org/" },
+    { 12622,	"Devicescape",		"http://www.devicescape.com/" },
     {    0,	NULL,			NULL },
 };
 
@@ -103,8 +105,8 @@ scli_get_vendor_oid(guint32 oid[], guint16 len)
  * The following list is taken from the Ethernet code list
  * available from <ftp://ftp.cavebear.com/pub/Ethernet-codes>:
  *
- * $Revision: 742 $
- * $Date: 2005-04-14 00:20:30 +0200 (Thu, 14 Apr 2005) $
+ * $Revision: 2887 $
+ * $Date: 2008-06-20 22:09:19 +0200 (Fri, 20 Jun 2008) $
  *
  * The list was subsequently cleaned up and modified to suit the
  * needs of the scli package.
